module Graal

   class Log < Enumerator

      def initialize(backend, revision, path, opts = {})
         super() do |yielder|
            commits_id = @backend.log(revision, path, opts)
            commits_id.each { |cid| yielder << Commit.new(@backend, cid) }
         end

         @backend = backend
         @revision = revision
         @path = path
         @opts = opts
      end

      def number(n)
         push_opt('max-count', n)
      end
      alias n         number
      alias max_count number

      def skip(n)
         push_opt('skip', n)
      end

      def since(time)
         push_opt('since', time_pattern_for(time))
      end
      alias after since

      def until(time)
         push_opt('until', time_pattern_for(time))
      end
      alias before until

      def author(actor)
         push_opt('author', actor_pattern_for(actor))
      end

      def committer(actor)
         push_opt('committer', actor_pattern_for(actor))
      end

   private

      def push_opt(name, value = true)
         Log.new(@backend, @revision, @path, @opts.merge(name => value))
      end

      def time_pattern_for(time_or_pattern)
         return time_or_pattern.to_i if time_or_pattern.is_a? Time
      end

      def actor_pattern_for(actor_or_pattern)
         return actor_or_pattern.header_line if actor_or_pattern.is_a? Actor
         actor_or_pattern
      end
   end
end
