require 'graal/tree_blob'

module Graal

   class Tree < TreeBlob

      def child(child_path)
         fullpath = File.join(@path, child_path)
         fullpath[ 0] = '' if fullpath[ 0] == ?/
         fullpath[-1] = '' if fullpath[-1] == ?/
         type, child_gitid = @backend.child(gitid, fullpath)
         child_for(type, fullpath, child_gitid)
      end
      alias [] child
      alias /  child

      def children(&block)
         enum_children(:children, &block)
      end
      alias each_child children
      alias list       children
      alias ls         children

      def trees(&block)
         enum_children(:trees, 'tree', &block)
      end
      alias each_tree  trees
      alias list_trees trees

      def blobs(&block)
         enum_children(:blobs, 'blob', &block)
      end
      alias each_blob  blobs
      alias list_blobs blobs

   private

      def enum_children(method, filter = nil)
         dirpath = @path.empty? ? path : path + '/'
         @backend.children(gitid, dirpath, filter).each do |data|
            yield child_for(*data)
         end if block_given?
         Enumerator.new(self, method)
      end

      def child_for(type, fullpath, gitid)
         return Blob.new(@backend, @revision, fullpath, gitid) if type == 'blob'
         Tree.new(@backend, @revision, fullpath, gitid)
      end
   end
end
