require 'graal/log'

module Graal

   class TreeBlob

      attr_reader :revision, :path

      def initialize(backend, revision, path, gitid = nil)
         @backend = backend
         @revision = revision || 'master'
         @path = path || ''
         @gitid = gitid
      end

      def gitid
         @gitid ||= @backend.gitid_for(@revision, @path)
      end
      alias git_hash gitid

      def abbrev(n = 8)
         gitid[0...n]
      end

      def tree?
         is_a? Tree
      end

      def blob?
         is_a? Blob
      end

      def log
         Log.new(@backend, @revision, @path)
      end

      def history(n = nil, skip = nil)
         history = log
         history = log.number(n)  unless n.nil?
         history = log.skip(skip) unless skip.nil?
         history.to_a
      end
   end
end
